"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deepTransform = exports.makeTransform = exports.camelToSnake = exports.snakeToCamel = void 0;
/* eslint-disable consistent-return */
const validate_1 = require("./validate");
const snakeToCamel = (str) => str.replace(/([_-]\w)/g, (m) => m[1].toUpperCase());
exports.snakeToCamel = snakeToCamel;
const camelToSnake = (str) => str
    .replace(/[\w]([A-Z])/g, (m) => `${m[0]}_${m[1]}`)
    .replace(/[A-Z]([A-Z])/g, (m) => `${m[0]}_${m[1]}`)
    .toLowerCase();
exports.camelToSnake = camelToSnake;
const makeTransform = (meta, opts = {}) => {
    const { toJSON } = opts;
    const target = 'fnTarget';
    const body = `return { \n ${toJSON
        ? Object.entries(meta)
            .map(([key, value]) => `${key}: Array.isArray(${target}.${value}) ? 
              ${target}.${value}.map(e => typeof e === 'object' && e !== null && e.toJSON instanceof Function ? e.toJSON() : e) :
              typeof ${target}.${value} === 'object' && ${target}.${value} !== null && ${target}.${value}.toJSON instanceof Function ?
              ${target}.${value}.toJSON() :
              ${target}.${value}`)
            .join(',\n')
        : Object.entries(meta)
            .map(([key, value]) => `${key}: ${target}.${value}`)
            .join(',\n')} };`;
    // eslint-disable-next-line @typescript-eslint/no-implied-eval
    return new Function(target, body);
};
exports.makeTransform = makeTransform;
const deepTransform = (target, handler, mutate, skip) => {
    if (!target)
        return;
    if (skip && target[skip]) {
        // eslint-disable-next-line no-param-reassign
        delete target[skip];
        return target;
    }
    return Object.entries(target).reduce((acc, [key, val]) => {
        const newKey = handler(key);
        if (Array.isArray(val)) {
            const newVal = val.map((e) => {
                if (typeof e !== 'object' || validate_1.isObjectId(e)) {
                    return e;
                }
                return exports.deepTransform(e, handler, mutate, skip);
            });
            acc[newKey] = newVal;
            mutate && key !== newKey && delete acc[key];
            return acc;
        }
        if (typeof val === 'object') {
            if (val === null || !Object.keys(val).length || validate_1.isObjectId(val)) {
                acc[newKey] = val;
                mutate && key !== newKey && delete acc[key];
                return acc;
            }
            const newVal = exports.deepTransform(val, handler, mutate, skip);
            acc[newKey] = newVal;
            mutate && key !== newKey && delete acc[key];
            return acc;
        }
        acc[newKey] = val;
        mutate && key !== newKey && delete acc[key];
        return acc;
    }, mutate ? target : {});
};
exports.deepTransform = deepTransform;
